######################################
#                                    #
#      Author : Fabien Collet        #
#   Website : www.fabiencollet.com   #
#                                    #
######################################


from PySide import QtGui
from PySide import QtCore
from random import uniform
import maya.cmds as mc


class MyDoubleSpinBox(QtGui.QDoubleSpinBox):
    def __init__(self, initValue):
        super(MyDoubleSpinBox, self).__init__()

        self.setRange(-9999.9, 9999.9)
        self.setValue(initValue)
        self.setDecimals(3)
        self.setSingleStep(0.250)
        self.setAlignment(QtCore.Qt.AlignCenter)


class VectorSpinBox():

    def __init__(self, initValue):

        self.layout = QtGui.QHBoxLayout()

        self.x = MyDoubleSpinBox(initValue)
        self.y = MyDoubleSpinBox(initValue)
        self.z = MyDoubleSpinBox(initValue)

        self.uniform = QtGui.QLineEdit()
        self.uniform.setFixedWidth(25)
        self.uniform.setAlignment(QtCore.Qt.AlignCenter)
        self.uniform.returnPressed.connect(self.changedVector)

        self.layout.addWidget(self.uniform)
        self.layout.addWidget(self.x)
        self.layout.addWidget(self.y)
        self.layout.addWidget(self.z)

    def getValue(self):
        valueX = self.x.value()
        valueY = self.y.value()
        valueZ = self.z.value()
        return valueX, valueY, valueZ

    def changedVector(self):

        newValue = self.uniform.text()

        self.x.setValue(float(newValue))
        self.y.setValue(float(newValue))
        self.z.setValue(float(newValue))

        self.uniform.clear()

    def disable(self):
        self.uniform.setEnabled(False)
        self.x.setEnabled(False)
        self.y.setEnabled(False)
        self.z.setEnabled(False)

    def enable(self):
        self.uniform.setEnabled(True)
        self.x.setEnabled(True)
        self.y.setEnabled(True)
        self.z.setEnabled(True)


class RandomizerLabel():

    def __init__(self, name):

        self.layout = QtGui.QHBoxLayout()

        self.checkBox = QtGui.QCheckBox()
        self.checkBox.setChecked(True)
        self.checkBox.setFixedWidth(25)

        self.x = QtGui.QLabel(name + ' X')
        self.y = QtGui.QLabel(name + ' Y')
        self.z = QtGui.QLabel(name + ' Z')

        self.x.setAlignment(QtCore.Qt.AlignCenter)
        self.y.setAlignment(QtCore.Qt.AlignCenter)
        self.z.setAlignment(QtCore.Qt.AlignCenter)

        self.layout.addWidget(self.checkBox)
        self.layout.addWidget(self.x)
        self.layout.addWidget(self.y)
        self.layout.addWidget(self.z)


class RandomizerUi(QtGui.QDialog):

    def __init__(self):
        super(RandomizerUi, self).__init__()

        self.setWindowTitle('Randomizer')
        self.setMinimumSize(300,300)
        self.setWindowState(QtCore.Qt.WindowStaysOnTopHint)

        self.oldTransform = {}

    def createLayout(self):

        # Main Layout
        self.mainLayout = QtGui.QVBoxLayout()

        # Settings Layout
        self.settingLayout = QtGui.QVBoxLayout()

        self.translateLayout = QtGui.QVBoxLayout()
        self.rotateLayout = QtGui.QVBoxLayout()
        self.scaleLayout = QtGui.QVBoxLayout()

        # Button Layout
        self.buttonLayout = QtGui.QHBoxLayout()

    def createWidget(self):

        self.translateLabel = RandomizerLabel('Translate')

        self.translateMinVector = VectorSpinBox(-1)
        self.translateMaxVector = VectorSpinBox(1)

        self.rotateLabel = RandomizerLabel('Rotate')

        self.rotateMinVector = VectorSpinBox(-1)
        self.rotateMaxVector = VectorSpinBox(1)

        self.scaleLabel = RandomizerLabel('Scale')

        self.scaleMinVector = VectorSpinBox(-1)
        self.scaleMaxVector = VectorSpinBox(1)

        # Buttons
        self.processBtn = QtGui.QPushButton('Process')
        self.undoBtn = QtGui.QPushButton('Undo')

    def addWidgetToLayout(self):

        # Buttons
        self.buttonLayout.addWidget(self.processBtn)
        self.buttonLayout.addWidget(self.undoBtn)

    def createConnections(self):

        # CheckBox
        self.translateLabel.checkBox.clicked.connect(self.translateCheck)
        self.rotateLabel.checkBox.clicked.connect(self.rotateCheck)
        self.scaleLabel.checkBox.clicked.connect(self.scaleCheck)

        # Buttons
        self.processBtn.clicked.connect(self.process)
        self.undoBtn.clicked.connect(self.reset)

    def createLayoutHierarchy(self):

        # Translate
        self.translateLayout.addLayout(self.translateLabel.layout)
        self.translateLayout.addLayout(self.translateMinVector.layout)
        self.translateLayout.addLayout(self.translateMaxVector.layout)

        # Rotate
        self.rotateLayout.addLayout(self.rotateLabel.layout)
        self.rotateLayout.addLayout(self.rotateMinVector.layout)
        self.rotateLayout.addLayout(self.rotateMaxVector.layout)

        # Scale
        self.scaleLayout.addLayout(self.scaleLabel.layout)
        self.scaleLayout.addLayout(self.scaleMinVector.layout)
        self.scaleLayout.addLayout(self.scaleMaxVector.layout)


        # Setting Layout

        self.settingLayout.addLayout(self.translateLayout)

        self.settingLayout.addLayout(self.rotateLayout)

        self.settingLayout.addLayout(self.scaleLayout)

        # Main Layout
        self.mainLayout.addLayout(self.settingLayout)
        self.mainLayout.addLayout(self.buttonLayout)

        # Window Layout
        self.setLayout(self.mainLayout)

    def displayUi(self):

        self.createLayout()
        self.createWidget()
        self.addWidgetToLayout()
        self.createConnections()
        self.createLayoutHierarchy()
        self.show()

    def translateCheck(self):
        check = self.translateLabel.checkBox.isChecked()
        if check == True:
            self.translateMinVector.enable()
            self.translateMaxVector.enable()
        else:
            self.translateMinVector.disable()
            self.translateMaxVector.disable()

    def rotateCheck(self):
        check = self.rotateLabel.checkBox.isChecked()
        if check == True:
            self.rotateMinVector.enable()
            self.rotateMaxVector.enable()
        else:
            self.rotateMinVector.disable()
            self.rotateMaxVector.disable()

    def scaleCheck(self):
        check = self.scaleLabel.checkBox.isChecked()
        if check == True:
            self.scaleMinVector.enable()
            self.scaleMaxVector.enable()
        else:
            self.scaleMinVector.disable()
            self.scaleMaxVector.disable()

    def process(self):
        translateCheck = self.translateLabel.checkBox.isChecked()
        rotateCheck = self.rotateLabel.checkBox.isChecked()
        scaleCheck = self.scaleLabel.checkBox.isChecked()

        selectionList = mc.ls(selection=True)
        if selectionList:
            for obj in selectionList:
                if mc.objectType(obj) == 'transform':

                    oldTX = mc.getAttr('%s.translateX' % obj)
                    oldTY = mc.getAttr('%s.translateY' % obj)
                    oldTZ = mc.getAttr('%s.translateZ' % obj)

                    oldRX = mc.getAttr('%s.rotateX' % obj)
                    oldRY = mc.getAttr('%s.rotateY' % obj)
                    oldRZ = mc.getAttr('%s.rotateZ' % obj)

                    oldSX = mc.getAttr('%s.scaleX' % obj)
                    oldSY = mc.getAttr('%s.scaleY' % obj)
                    oldSZ = mc.getAttr('%s.scaleZ' % obj)

                    self.oldTransform[obj] = [oldTX, oldTY, oldTZ, oldRX, oldRY, oldRZ, oldSX, oldSY, oldSZ]

                    if translateCheck:
                        tMin = self.translateMinVector.getValue()
                        tMax = self.translateMaxVector.getValue()

                        randTX = uniform(tMin[0],tMax[0])
                        randTY = uniform(tMin[1],tMax[1])
                        randTZ = uniform(tMin[2],tMax[2])

                        newTX = oldTX + randTX
                        newTY = oldTY + randTY
                        newTZ = oldTZ + randTZ

                        mc.xform(obj, translation=(newTX, newTY, newTZ))

                    if rotateCheck:
                        rMin = self.rotateMinVector.getValue()
                        rMax = self.rotateMaxVector.getValue()

                        randRX = uniform(rMin[0], rMax[0])
                        randRY = uniform(rMin[1], rMax[1])
                        randRZ = uniform(rMin[2], rMax[2])


                        newRX = oldRX + randRX
                        newRY = oldRY + randRY
                        newRZ = oldRZ + randRZ

                        mc.xform(obj, rotation=(newRX, newRY, newRZ))

                    if scaleCheck:
                        sMin = self.scaleMinVector.getValue()
                        sMax = self.scaleMaxVector.getValue()

                        randSX = uniform(sMin[0], sMax[0])
                        randSY = uniform(sMin[1], sMax[1])
                        randSZ = uniform(sMin[2], sMax[2])

                        newSX = oldSX + randSX
                        newSY = oldSY + randSY
                        newSZ = oldSZ + randSZ

                        mc.xform(obj, scale=(newSX, newSY, newSZ))

                else:
                    print '[WARNING][TYPE][NOT_TRANSFORM]   ',obj
        else:
            print '[WARNING][NOTHING][SELECTED]   '

    def reset(self):
        if self.oldTransform:
            for i in self.oldTransform:
                mc.xform(i,
                         translation=(
                             self.oldTransform[i][0],
                             self.oldTransform[i][1],
                             self.oldTransform[i][2]
                         ),
                         rotation=(
                             self.oldTransform[i][3],
                             self.oldTransform[i][4],
                             self.oldTransform[i][5]
                         ),
                         scale=(
                             self.oldTransform[i][6],
                             self.oldTransform[i][7],
                             self.oldTransform[i][8]
                         ))


randomizer = RandomizerUi()
randomizer.displayUi()